var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Alert, Linking, ScrollView, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { sprintf } from 'sprintf-js';
import { loadTouchState } from '../../../actions/TouchActions';
import { getAppConfig } from '../../../common/appConfig';
import { lstrings } from '../../../common/locales/strings';
import * as Constants from '../../../constants/index';
import { useHandler } from '../../../hooks/useHandler';
import { useImports } from '../../../hooks/useImports';
import { useScrollToEnd } from '../../../hooks/useScrollToEnd';
import { enableTouchId } from '../../../keychain';
import { useDispatch } from '../../../types/ReduxTypes';
import { ChallengeModal } from '../../modals/ChallengeModal';
import { Airship, showError } from '../../services/AirshipInstance';
import { useTheme } from '../../services/ThemeContext';
import { Checkbox } from '../../themed/Checkbox';
import { EdgeText } from '../../themed/EdgeText';
import { Fade } from '../../themed/Fade';
import { MainButton } from '../../themed/MainButton';
import { ThemedScene } from '../../themed/ThemedScene';
const TosComponent = (props) => {
    const { branding, hidePasswordTerms, onBack, onNext } = props;
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    const [termValues, setTermValues] = React.useState(hidePasswordTerms ? [false, false, false] : [false, false, false, false]);
    const showNext = !termValues.includes(false);
    const scrollViewRef = useScrollToEnd(showNext);
    const buttonType = theme.preferPrimaryButton ? 'primary' : 'secondary';
    const { appName = lstrings.app_name_default } = branding;
    const terms = [
        sprintf(lstrings.terms_one, appName),
        hidePasswordTerms ? lstrings.terms_two_alt : lstrings.terms_two,
        ...(hidePasswordTerms ? [] : [sprintf(lstrings.terms_three, appName)]),
        sprintf(lstrings.terms_four, appName)
    ];
    const handleStatusChange = useHandler((index, value) => {
        const newTermValues = [...termValues];
        newTermValues[index] = value;
        setTermValues(newTermValues);
    });
    const handleNextPress = useHandler(() => {
        onNext().catch((e) => {
            console.error(e);
            Alert.alert(lstrings.create_account_error_title, lstrings.create_account_error_message + '\n' + e.message, [{ text: lstrings.ok }]);
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'newAccountUsername', params: {} }
            });
        });
    });
    return (React.createElement(ThemedScene, { onBack: onBack, title: lstrings.account_confirmation },
        React.createElement(ScrollView, { ref: scrollViewRef, contentContainerStyle: styles.content },
            React.createElement(EdgeText, { style: styles.subtitle }, `${lstrings.review}: ${lstrings.read_understod_2}`),
            terms.map((term, index) => (React.createElement(Checkbox, { key: index, textStyle: styles.term, value: termValues[index], onChange: (value) => handleStatusChange(index, value), marginRem: [0, 0, 1.33, 0] }, term))),
            React.createElement(EdgeText, { style: styles.agreeText, numberOfLines: 2, onPress: () => __awaiter(void 0, void 0, void 0, function* () { return yield Linking.openURL(getAppConfig().termsOfServiceSite); }) },
                lstrings.read_understod_1,
                React.createElement(EdgeText, { style: styles.agreeTextLink }, lstrings.read_understod_2)),
            React.createElement(View, { style: styles.actions },
                React.createElement(Fade, { visible: showNext },
                    React.createElement(MainButton, { label: lstrings.confirm, paddingRem: 0.7, type: buttonType, onPress: handleNextPress }))))));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(0.5)
    },
    subtitle: {
        fontFamily: theme.fontFaceBold,
        color: theme.secondaryText,
        fontSize: theme.rem(1),
        marginBottom: theme.rem(1.5)
    },
    term: {
        fontSize: theme.rem(0.875)
    },
    agreeText: {
        width: '60%',
        alignSelf: 'center',
        marginTop: theme.rem(1),
        marginHorizontal: theme.rem(2),
        fontSize: theme.rem(0.875)
    },
    agreeTextLink: {
        fontSize: theme.rem(0.875),
        color: theme.linkText
    },
    actions: {
        flexDirection: 'row',
        justifyContent: 'center',
        marginTop: theme.rem(2.75),
        minHeight: theme.rem(6)
    }
}));
const setTouchOtp = (account, dispatch) => __awaiter(void 0, void 0, void 0, function* () {
    yield enableTouchId(account).catch(e => {
        console.log(e); // Fail quietly
    });
    yield account.dataStore.setItem(Constants.OTP_REMINDER_STORE_NAME, Constants.OTP_REMINDER_KEY_NAME_CREATED_AT, Date.now().toString());
    dispatch(loadTouchState());
});
export const NewAccountTosScene = (props) => {
    const { route, branding } = props;
    const imports = useImports();
    const { context, accountOptions, experimentConfig, onLogin, onLogEvent = (event, values) => { } } = imports;
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'newAccountPin', params: Object.assign({}, route.params) }
        });
    });
    const handleNext = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        const { username, password, pin } = route.params;
        const lightAccount = experimentConfig.createAccountType === 'light';
        if (experimentConfig.signupCaptcha === 'withCaptcha') {
            onLogEvent('Signup_Captcha_Shown');
            const result = yield Airship.show(bridge => {
                return React.createElement(ChallengeModal, { bridge: bridge });
            });
            // User closed the modal
            if (result == null) {
                onLogEvent('Signup_Captcha_Quit');
                return;
            }
            if (!result) {
                onLogEvent('Signup_Captcha_Failed');
                showError(lstrings.failed_captcha_error);
                return;
            }
            onLogEvent('Signup_Captcha_Passed');
        }
        let error;
        try {
            dispatch({
                type: 'NAVIGATE',
                data: {
                    name: 'newAccountWait',
                    params: {
                        title: lstrings.great_job,
                        message: lstrings.hang_tight + '\n' + lstrings.secure_account
                    }
                }
            });
            const account = yield context.createAccount(Object.assign(Object.assign({}, accountOptions), { username,
                password,
                pin }));
            account.watch('loggedIn', loggedIn => {
                if (!loggedIn)
                    dispatch({ type: 'RESET_APP' });
            });
            yield setTouchOtp(account, dispatch);
            if (lightAccount) {
                if (onLogin == null) {
                    console.error('NewAccountTosScene: onLogin required for light accounts');
                    return;
                }
                onLogin(account);
            }
            else {
                dispatch({
                    type: 'NAVIGATE',
                    data: {
                        name: 'newAccountReview',
                        params: Object.assign(Object.assign({}, route.params), { account })
                    }
                });
            }
        }
        catch (e) {
            showError(e);
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'newAccountTos', params: route.params }
            });
            error = String(e);
        }
        onLogEvent('Signup_Terms_Agree_and_Create_User', {
            error
        });
    }));
    return (React.createElement(TosComponent, { hidePasswordTerms: route.params.username == null, branding: branding, onBack: handleBack, onNext: handleNext }));
};
export const UpgradeTosScene = (props) => {
    const { route, branding } = props;
    const { onLogEvent = (event, values) => { } } = useImports();
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'upgradeUsername', params: Object.assign({}, route.params) }
        });
    });
    const handleNext = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        const { account, username, password } = route.params;
        let error;
        try {
            if (username == null || password == null)
                throw new Error('Failed to update account, missing username or password');
            yield account.changeUsername({
                username,
                password
            });
            dispatch({
                type: 'NAVIGATE',
                data: {
                    name: 'upgradeAccountReview',
                    params: Object.assign({}, route.params)
                }
            });
        }
        catch (e) {
            error = String(e);
        }
        onLogEvent('Backup_Terms_Agree_and_Create_User', { error });
    }));
    return (React.createElement(TosComponent, { branding: branding, hidePasswordTerms: false, onBack: handleBack, onNext: handleNext }));
};
